﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace FileSystem {
  public abstract class Entry { // the base name of the composite type

     protected string name;    //  the entry's name; accessible to the subclass code

     // get and set the value of name:
     public string Name {
       get { return name; }
       set { name = value; }
     }

     // returns a string that is the name of this Entry 
     //  ( if the Entry is a folder, will also list the names of all subEntries )
     public override string ToString() { return makeString(""); }

     // Each subclass must provide the code for this helper method for making 
     //   their own string representation.   THIS IS "VISITOR-DESIGN PATTERN" STYLE.
     // Param: indentation -- how many leading spaces to print before the string
     // Returns: the indentation + the string representation of this Entry
     public abstract string makeString(string indentation);
     // { this method is coded in each subclass of Entry }


     /*  ANOTHER STYLE OF CODING  makeString --- "EXPLICIT RECURSION STYLE": 
     public string makeString(string indent) {
       string ans = "";
       if (this is TextFile) { // was  this  object constructed from class TextFile ?
         ans = indent + "Text file: " + this.name + "\r\n";
       }
       else if (this is Shortcut) {
         ans = indent + "Shortcut: " + this.name + "\r\n";
       }
       else if (this is Folder) {
         ans = indent + "Folder:  " + this.name + " containing:\r\n";
         List<Entry> contents = ((Folder)this).getListing(); // get all entries in folder
         foreach (Entry entry in contents) {
           ans = ans + entry.makeString(indent + "   ");
         }
         ans = ans + indent + "End Folder " +  this.name + "\r\n";
       }
       return ans;
     }
   */

  }
}
