﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using FileSystem;

namespace UnitTests {
  class Program {

    // (i) make a file system:
    static Folder filesystem = makeSampleSystem();  // see this method at the end of this file

    // (ii) run some tests:
    static void Main(string[] args) {

      Console.WriteLine("FILE SYSTEM CONTENTS:");  Console.WriteLine();
      Console.WriteLine(filesystem.ToString());

      // Since  a.txt  is saved in the Root filesystem, we can fetch it and print it:
      Console.WriteLine("FIND AND PRINT contents of  a.txt,  saved in Root filesystem:");
      Entry atxt = filesystem.find("a.txt");
      Console.WriteLine(atxt.ToString());
      Console.WriteLine(((TextFile)atxt).Lines);
      Console.WriteLine();

      // here are tests for searching for entries using directory paths:
      Console.WriteLine("USE SOME PATHS TO SEARCH FOR ENTRIES:"); Console.WriteLine();
      test("a.txt");
      test("F/b.txt");
      test("F/G");
      test("F/G/c.txt");
      Console.WriteLine("This one should be an invalid entry:");
      test("Bad/x.txt");
      Console.WriteLine("Try this one last, it's not required for this exercise:");
      test("F/JumpTo.a");

      Console.ReadLine();
    }

    // helper method that tests how to  search 
    // Param:  path - states the path to search in the filesystem, e.g. "F/G/x.txt"
    // The method prints the Entry found by the  search  for  path
    static void test(string path) {
        Console.WriteLine("Search for " + path);
        Entry e = filesystem.search(path);  // test the coding of  search  here
        if (e != null) {
            Console.WriteLine(e.ToString());
            if (e is TextFile) {
                Console.WriteLine(((TextFile)e).Lines);
            }
        }
        else { Console.WriteLine("invalid entry"); }
        Console.WriteLine();
    }


    // assembles a sample filesystem whose root folder is named "Root"; returns its handle.
    static Folder makeSampleSystem() {
      Folder root = new Folder("Root");

      TextFile a = new TextFile("a.txt");
      a.addLine("I am line 1 of a.txt"); a.addLine("I am line 2.");
      root.add(a);

      Folder f = new Folder("F");
      root.add(f);

      TextFile b = new TextFile("b.txt");
      b.addLine("I am file b.txt.  The end.");
      f.add(b);

      f.add(new Shortcut("JumpTo.a", a));   // add shortcut to "a.txt" to folder "F"

      Folder g = new Folder("G");
      f.add(g);

      TextFile c = new TextFile("c.txt");
      c.addLine("I am file c.txt.");
      g.add(c);

      Folder h = new Folder("H");
      root.add(h);

      return root;
    }
    
  }
}
